/***********************************************************************

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/
package gecco.game.dke;

import gecco.server.visibility.Point;
import gecco.server.core.*;
import java.util.Iterator;
import java.util.Collection;
import java.util.Vector;



/**
 * Referee.java
 *
 *
 * Created: Fri Jan 17 10:56:31 2003
 *
 * @author <a href="mailto:hans.sandstrom@fhs.mil.se">Hans Sandstrm</a>
 * @version
 */

public class DKEReferee extends gecco.server.referee.Referee {

    boolean blueObjectiveObtained = false;
    boolean redObjectiveObtained = false;
    boolean isVictoryPointsFound = false;
    boolean isGameEnded = false;
    VictoryPoint blueVictoryPoint;
    VictoryPoint redVictoryPoint;
    Point blueVPPos;
    Point redVPPos;
    boolean timeIsUp = false;
    int winningForce = Def.NO_FORCE;
    int blueScore = 0;
    int blueScorePointPoints = 0;
    int blueWarUnitsPoints = 0;
    int redScore = 0;
    int redScorePointPoints = 0;
    int redWarUnitsPoints = 0;
    

    public DKEReferee(){
    }

    public void startReferee(){
	new RefereeTimerThread(this, Def.MAX_GAME_TIME, Def.MAX_GAME_TIME);
    }

    public void timeIsUp(double time){
	if (time == Def.MAX_GAME_TIME) {
	    gameOver();
	}
    }

    public void newOwnerOfGameEventPoint(GameEventPoint gep){
	if (gep instanceof VictoryPoint) {
	    VictoryPoint vp = (VictoryPoint) gep;
	    int force = vp.getObjectiveFor();
	    if (force == Def.BLUE_FORCE) {
		if (vp.isObjectiveObtained()) {
		    blueObjectiveObtained = true;
		}
		else {
		    blueObjectiveObtained = false;
		}
	    }
	    else if (force == Def.RED_FORCE){
		if (vp.isObjectiveObtained()) {
		    redObjectiveObtained = true;
		}
		else {
		    redObjectiveObtained = false;
		}
	    }
	    isGameWon();
	}
    }

    public void findVictoryPoints(){
	Iterator it = ReferenceHolder.getUnitManager().getAllUnits().iterator();
	while (it.hasNext()) {
	    Unit unit = (Unit) it.next();
	    if (unit instanceof VictoryPoint) {
		VictoryPoint vp = (VictoryPoint) unit;
		int objectiveFor = vp.getObjectiveFor();
		if (objectiveFor == Def.BLUE_FORCE) {
		    blueVictoryPoint = vp;
		}
		if (objectiveFor == Def.RED_FORCE) {
		    redVictoryPoint = vp;
		}
	    }
	}
	isVictoryPointsFound = true;
	blueVPPos = new Point(blueVictoryPoint.getIntX(), blueVictoryPoint.getIntY());
	redVPPos = new Point(redVictoryPoint.getIntX(), redVictoryPoint.getIntY());
    }

    public synchronized boolean isGameWon(){
	if (!isGameEnded) {
	    if (!isVictoryPointsFound) {
		findVictoryPoints();  
	    }	
	    if (blueObjectiveObtained == redObjectiveObtained) {
		return false;
	    }
	    if (blueObjectiveObtained) {
		if (RouteFinder.isRouteOpen(redVPPos, blueVPPos, Def.BLUE_FORCE)) {
		    winningForce = Def.BLUE_FORCE;
		    gameOver();
		    return true;
		}
		else {
		    return false;
		}
	    }
	    else {
		if (RouteFinder.isRouteOpen(blueVPPos, redVPPos, Def.RED_FORCE)) {
		    winningForce = Def.RED_FORCE;
		    gameOver();
		    return true;
		}
		else {
		    return false;
		}
	    }
	}
	else {
	    return true;
	}
    }
	
    public void lastVisitorChanged(int newVisitor){
	if (blueObjectiveObtained != redObjectiveObtained) {
	    if (newVisitor == Def.BLUE_FORCE) {
		if (blueObjectiveObtained && !redObjectiveObtained) {
		    isGameWon();
		}
	    }
	    else if (newVisitor == Def.RED_FORCE) {
		if (redObjectiveObtained && !blueObjectiveObtained) {
		    isGameWon();
		}
	    }
	}
    }
    
    public void testForWinnerOnPoints(){
	if ((blueScore - redScore) >= Def.SCORE_ADVANTAGE_TO_WIN) {
	    winningForce = Def.BLUE_FORCE;  
	}
	if ((redScore - blueScore) >= Def.SCORE_ADVANTAGE_TO_WIN) {
	    winningForce = Def.RED_FORCE;
	}
    }

    private void calculateScore(){
	blueScore = 0;
	redScore = 0;
	calculateScoreFromScorePoints();
	calculateScoreFromWarUnits();
	blueScore = blueScorePointPoints + blueWarUnitsPoints;
	redScore = redScorePointPoints + redWarUnitsPoints;
    }


    private void calculateScoreFromWarUnits(){
	redWarUnitsPoints = 0;
	blueWarUnitsPoints = 0;
	Collection units = ReferenceHolder.getUnitManager().getAllUnits();
	Vector warUnits = filterOutWarUnits(units);
	Iterator it = warUnits.iterator();
	while (it.hasNext()) {
	    WarUnit wu = (WarUnit) it.next();
	    int force = wu.getForce();
	    if (force == Def.BLUE_FORCE) {
		blueWarUnitsPoints += wu.getHealth();
	    }
	    else if (force == Def.RED_FORCE){
		redWarUnitsPoints += wu.getHealth();
	    }
	}
    }

    private Vector filterOutWarUnits(Collection c){
	Vector returnVector = new Vector();
	Iterator it = c.iterator();
	while (it.hasNext()) {
	    Object o = it.next();
	    if (o instanceof WarUnit) {
		returnVector.add(o);
	    }
	}
	return returnVector;
    }

    private Vector filterOutScorePoints(Collection c){
	Iterator it = c.iterator();
	Vector v = new Vector();
	while (it.hasNext()) {
	    Object o = it.next();
	    if (o instanceof ScorePoint) {
		v.add(o);
	    }
	}
	return v;
    }

    private Vector filterOutVictoryPoints(Collection c){
	Iterator it = c.iterator();
	Vector v = new Vector();
	while (it.hasNext()) {
	    Object o = it.next();
	    if (o instanceof VictoryPoint) {
		v.add(o);
	    }
	}
	return v;
    }

    private void calculateScoreFromScorePoints(){
	blueScorePointPoints = 0;
	redScorePointPoints = 0;
	Collection units = ReferenceHolder.getUnitManager().getAllUnits();
	Vector sortedScorePoints = sortScorePoints(filterOutScorePoints(units));
	Iterator it1 = sortedScorePoints.iterator();
	while (it1.hasNext()) {
	    Vector v = (Vector) it1.next();
	    if (!v.isEmpty()) {
		Iterator it2 = v.iterator();
		ScorePoint sp = (ScorePoint) it2.next();
		int force = sp.getOwner();
		boolean oneOwner = true;
		while (it2.hasNext()) {
		    if (force != ((ScorePoint) it2.next()).getOwner()) {
			oneOwner = false;
		    }
		}
		if (oneOwner) {
		    if (force == Def.BLUE_FORCE) {
			blueScorePointPoints += sp.getValue();
		    } 
		    else if (force == Def.RED_FORCE) {
			redScorePointPoints += sp.getValue();
		    }
		}
	    }
	}
    }
	
    private Vector sortScorePoints(Vector inputVector){
	Vector returnVector = new Vector();
	while (!inputVector.isEmpty()) {
	    Vector v = new Vector();
	    returnVector.add(v);
	    ScorePoint sp = (ScorePoint) inputVector.get(0);
	    String name = sp.getName();
	    Iterator it = inputVector.iterator();
	    while (it.hasNext()) { // sort out objects with  name as first object in vector
		ScorePoint sp2 = (ScorePoint) it.next();
		if (sp2.getName().equals(name)) {
		    v.add(sp2);
		}
	    }
	    it = v.iterator();
	    while (it.hasNext()) { // remove objects sorted out 
		inputVector.remove(it.next());
	    }
	}
	return returnVector;
    }

    private void stopGame(){
	isGameEnded = true;
	ReferenceHolder.getClock().freeze(true);
    }

    private void gameOver(){
	stopGame();
	calculateScore();
	if (winningForce ==  Def.NO_FORCE) {
	    testForWinnerOnPoints();
	}
	writeResult();
	String message = Def.getWinnerIsString() + Def.getForceName(winningForce);
	ReferenceHolder.getClient().gameOver(message);
	ReferenceHolder.out.println("Game stoped by referee, game over.");
    }

    private void coffeeBreak(){
	ReferenceHolder.getClock().freeze(true);
	ReferenceHolder.out.println("Game paused by referee, type 'start' to continue.");
	ReferenceHolder.getClient().coffeeBreak(Def.getCoffeeBreakMessage());
    }

    public void writeResult(){
	writeResult(ReferenceHolder.out);
	writeSpecifiedResult(ReferenceHolder.out);
	if (ReferenceHolder.log != null) {
	    writeResult(ReferenceHolder.log);
	    writeSpecifiedResult(ReferenceHolder.log);
	    writeUnitsSpecification(ReferenceHolder.log);
	}
    }
	
    private void writeResult(java.io.PrintWriter out){
	out.println("************ Game Result *************");
	out.println("Winner   : " + Def.getForceName(winningForce));
	out.println(Def.getForceName(Def.BLUE_FORCE) + " score: " + blueScore);
	out.println(Def.getForceName(Def.RED_FORCE) + " score: " + redScore);
	out.println("Game ended at " + ReferenceHolder.getTimeString().getGameTime() + " game time.");
	out.println("******** End of Game Result **********");
	out.println();
    }

    private void writeSpecifiedResult(java.io.PrintWriter out){
	out.println("************ Specified Result *************");
	out.println(Def.getForceName(Def.BLUE_FORCE) + " score from ScorePoints: " + blueScorePointPoints);
	out.println(Def.getForceName(Def.BLUE_FORCE) + " score from WarUnits   : " + blueWarUnitsPoints);
	out.println(Def.getForceName(Def.RED_FORCE) + " score from ScorePoints: " + redScorePointPoints);
	out.println(Def.getForceName(Def.RED_FORCE) + " score from WarUnits   : " + redWarUnitsPoints);
	out.println("******** End of Specified Result **********");
	out.println();
    }

    private void writeUnitsSpecification(java.io.PrintWriter out){
	out.println("************ War Units *************");
	Vector v = filterOutWarUnits(ReferenceHolder.getUnitManager().getAllUnits());
	Iterator it = v.iterator();
	while (it.hasNext()) {
	    Unit u = (Unit) it.next();
	    u.printWriteMe(out);
	    out.println();
	}
	out.println("************ End of War Units *************");
	out.println();
	out.println("************ Victory Points *************");
	v = filterOutVictoryPoints(ReferenceHolder.getUnitManager().getAllUnits());
	it = v.iterator();
	while (it.hasNext()) {
	    Unit u = (Unit) it.next();
	    u.printWriteMe(out);
	    out.println();
	}
	out.println("************ End of Victory Points *************");
	out.println();
	out.println("************ Score Points *************");
	v = filterOutScorePoints(ReferenceHolder.getUnitManager().getAllUnits());
	it = v.iterator();
	while (it.hasNext()) {
	    Unit u = (Unit) it.next();
	    u.printWriteMe(out);
	    out.println();
	}
	out.println("************ End of Score Points *************");
	out.println();
    }

}// DKEReferee
