/***********************************************************************

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/
package gecco.game.dke;

import gecco.server.core.*;
import java.util.NoSuchElementException;
import gecco.server.replaymanager.ReplayReady;
import java.io.ObjectOutputStream;

/**
 * GameEventPoint.java
 *
 *
 * Created: Wed Feb 05 13:43:31 2003
 *
 * @author <a href="mailto:Hans.Sandstrom@fhs.mil.se">Hans Sandstrm</a>
 * @version
 */

public class GameEventPoint extends Unit implements ReplayReady  {

    public GameEventPoint (){
	super();
    }
    
    public void automatonLastVisitorChanged(int newLastVisitor){
	changeOwner(newLastVisitor);
    }

    public int getOwner(){
	int owner;
	try {
	    owner = getIntProperty("Owner");
	} catch (NoSuchElementException e) {
	    owner = Def.NO_FORCE; 
	}
	return owner;
    }

    public void changeOwner(int newOwner){
	setProperty("Owner", newOwner);
	DKEReferee referee = (DKEReferee)ReferenceHolder.getReferee();
	referee.newOwnerOfGameEventPoint(this);
	// Don't record replay data when replaying
	if (!ReferenceHolder.inReplayMode()) {
	    Object[] parameters = {new Integer(newOwner)};
	    saveReplayData(CHANGE_OWNER, parameters);
	}
    }

    public String toString(){
	return super.toString() +
	    "\nOwner         : " + Def.getForceName(getOwner());
    }    

    public void printWriteMe(java.io.PrintWriter out){
	super.printWriteMe(out);
	out.println("Owner         : " + Def.getForceName(getOwner()));
    }
    // implementation of gecco.server.replaymanager.ReplayReady interface

    /**
     *
     * @param method <description>
     * @param parameters <description>
     */
    public void saveReplayData(int method, Object[] parameters){
	ObjectOutputStream replayLog = ReferenceHolder.getReplayLog();
	try {
	    replayLog.writeDouble(ReferenceHolder.getClock().gameTimeNow());
	    replayLog.writeInt(UNIT);
	    replayLog.writeInt(getHandle());
	    replayLog.writeInt(method);
	    replayLog.writeObject(parameters);
	    replayLog.flush();
	} catch (Exception e) {
	    e.printStackTrace();
	} 
    }

    public final int CHANGE_OWNER = 100;

    /**

     *
     * @param method <description>
     * @param parameters <description>
     */
    public void playReplayData(int method, Object[] parameters){
	if (method == CHANGE_OWNER) {
	    int n = ((Integer) parameters[0]).intValue();
	    changeOwner(n);
	}
	else {
	    // Debug info
	    System.out.println("WarUnit:playReplayData(int, Object[]) method " + method + " not handled!");
	}
    }

}// GameEventPoint
